public class Mocker : Gtk.Application
{
    private Gtk.ApplicationWindow window;
    private Mockup mockup;
    private MockupView view;
    private Gtk.ScrolledWindow controls_box;

    public Mocker ()
    {
        Object (application_id: "com.ubuntu.mocker", flags: ApplicationFlags.FLAGS_NONE);
    }

    public override void startup ()
    {
        base.startup ();

        window = new Gtk.ApplicationWindow (this);

        var vbox = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
        vbox.visible = true;
        window.add (vbox);

        var toolbar = new Gtk.Toolbar ();
        toolbar.visible = true;
        toolbar.get_style_context ().add_class (Gtk.STYLE_CLASS_PRIMARY_TOOLBAR);
        vbox.pack_start (toolbar, false, true, 0);

        var button = new Gtk.ToolButton.from_stock (Gtk.Stock.NEW);
        button.visible = true;
        toolbar.add (button);

        button = new Gtk.ToolButton.from_stock (Gtk.Stock.SAVE);
        button.visible = true;
        toolbar.add (button);

        mockup = new Mockup ();
        mockup.selected_item_changed.connect (selected_item_changed_cb);

        var hbox = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 0);
        hbox.visible = true;
        vbox.pack_start (hbox, true, true, 0);

        var sidebar_box = new Gtk.Box (Gtk.Orientation.VERTICAL, 5);
        sidebar_box.set_size_request (200, -1);
        sidebar_box.visible = true;
        hbox.pack_start (sidebar_box, false, true, 0);

        var item_box = new Gtk.Box (Gtk.Orientation.VERTICAL, 5);
        item_box.visible = true;
        sidebar_box.pack_start (item_box, false, true, 0);

        var item_button = new Gtk.Button.with_label ("Window");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockWindow ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Label");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockLabel ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Button");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockButton ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Check");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockCheck ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Radio");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockRadio ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Switch");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockSwitch ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Icon");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockIcon ()); });
        item_box.pack_start (item_button, false, true, 0);

        item_button = new Gtk.Button.with_label ("Note");
        item_button.visible = true;
        item_button.clicked.connect (() => { mockup.add_item (new MockNote ()); });
        item_box.pack_start (item_button, false, true, 0);

        controls_box = new Gtk.ScrolledWindow (null, null);
        sidebar_box.pack_start (controls_box, true, true, 0);

        view = new MockupView (mockup);
        view.visible = true;
        hbox.pack_start (view, true, true, 0);
    }

    public override void activate ()
    {
        window.visible = true;
    }

    private void selected_item_changed_cb ()
    {
        /* Remove old grid */
        if (controls_box.get_child () != null)
            controls_box.remove (controls_box.get_child ());

        var item = mockup.selected_item;
        if (item == null)
        {
            controls_box.visible = false;
            return;
        }

        controls_box.visible = true;

        var grid = new Gtk.Grid ();
        grid.visible = true;
        grid.row_spacing = 5;
        grid.column_spacing = 5;
        controls_box.add_with_viewport (grid);

        var row = 0;
        foreach (var prop in item.properties)
        {
            if (prop is MockItemStringProperty)
            {
                var p = prop as MockItemStringProperty;

                var label = new Gtk.Label ("%s:".printf (prop.description));
                label.set_alignment (0.0f, 0.5f);
                label.visible = true;
                grid.attach (label, 0, row, 2, 1);
                row++;

                var entry = new Gtk.Entry ();
                entry.text = p.value;
                entry.visible = true;
                entry.changed.connect (() => { p.value = entry.text; });
                grid.attach (entry, 0, row, 2, 1);
                row++;
            }
            else if (prop is MockItemBooleanProperty)
            {
                var p = prop as MockItemBooleanProperty;

                var label = new Gtk.Label (prop.description);
                label.set_alignment (0.0f, 0.5f);
                label.visible = true;
                grid.attach (label, 0, row, 1, 1);

                var s = new Gtk.Switch ();
                s.active = p.value;
                s.visible = true;
                s.notify["active"].connect (() => { p.value = s.active; });
                grid.attach (s, 1, row, 1, 1);
                row++;
            }
            else
            {
                warning ("Unknown property type");
            }
        }
    }
}

private void cairo_rounded_rectangle (Cairo.Context c, double x, double y, double width, double height, double radius)
{
    var w = width - radius * 2;
    var h = height - radius * 2;
    var kappa = 0.5522847498 * radius;
    c.move_to (x + radius, y);
    c.rel_line_to (w, 0);
    c.rel_curve_to (kappa, 0, radius, radius - kappa, radius, radius);
    c.rel_line_to (0, h);
    c.rel_curve_to (0, kappa, kappa - radius, radius, -radius, radius);
    c.rel_line_to (-w, 0);
    c.rel_curve_to (-kappa, 0, -radius, kappa - radius, -radius, -radius);
    c.rel_line_to (0, -h);
    c.rel_curve_to (0, -kappa, radius - kappa, -radius, radius, -radius);
}

public static int main (string[] args)
{
    Gtk.init (ref args);

    var mocker = new Mocker ();
    return mocker.run ();
}
